import 'dart:async';
import 'package:flutter/material.dart';
import 'dart:typed_data';

import '../consts.dart';
import '../widgets/main_widgets_container.dart';
import '../data_classes.dart';
import '../server_requests/fetch_markers.dart';
import '../server_requests/fetch_relation.dart';

class DefaultPage extends StatefulWidget {
  const DefaultPage({Key? key}) : super(key: key);

  @override
  _DefaultPageState createState() => _DefaultPageState();
}

class _DefaultPageState extends State<DefaultPage> {
  late Timer _timer;
  bool showBar = false;
  bool serverConnection = true;

  late Future<List<Measurement>> markerListFuture;
  late Future<Uint8List> relationFuture;

  @override
  void initState() {
    super.initState();
    // Try to fetch measurement data from server
    markerListFuture = fetchMeasurements().then((fetchResult) {
      List<Measurement> measurements = fetchResult.measurements;
      serverConnection = fetchResult.connected;

      // Return the measurements
      return measurements;
    }).catchError((error) {
      throw Exception("Failed to fetch measurements: $error");
    });


    relationFuture = fetchRelation();
    //relationFuture = Future.value(Uint8List(0));

    // Schedule fetchMarkerData to run periodically based on fetchInterval from consts
    const Duration interval = Duration(minutes: fetchInterval); // NB fetchInterval value to be determined
    _timer = Timer.periodic(interval, (timer) {
      fetchMeasurements();
    });
  }

  // Fetch-interval timer
  @override
  void dispose() {
    // Cancel timer on widget termination
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('IceMap'),
          actions: [
            IconButton(
              icon: const Icon(Icons.search),
              onPressed: () {
                setState(() {
                  showBar = !showBar;
                });
              },
            ),
          ],
        ),
        body: FutureBuilder(
          future: Future.wait([markerListFuture, relationFuture]),
          builder: (BuildContext context, AsyncSnapshot<List<dynamic>> snapshot) {
            // Display loading screen until data is fetched
            if (snapshot.connectionState == ConnectionState.waiting) {
              return Container(
                decoration: const BoxDecoration( // Background color for loading screen
                  color: darkestBlue,
                ),
                child: const Center(
                  child: Icon(
                    Icons.severe_cold, // Loading screen icon
                    color: lightBlue,
                    size: 100,
                  ),
                ),
              );
            } else if (snapshot.hasError) {
              return Center(child: Text('Error: ${snapshot.error}'));
            } else {
              if (!serverConnection) { // NB: implement alert
                print("Failed to connect to server");
              }
              // Display default page once all data is loaded from server
              List<Measurement> markerList = snapshot.data![0] as List<Measurement>;
              Uint8List relation = snapshot.data![1] as Uint8List;
              return Container( // Return container with list view and background color
                decoration: const BoxDecoration( // Set background color
                  gradient: LinearGradient(
                    begin: Alignment.topCenter,
                    end: Alignment.bottomCenter,
                    colors: [darkBlue, darkestBlue],
                  ),
                ),
                child: ListView(
                  children: [
                    MapContainerWidget(markerList: markerList, relation: relation),
                  ],
                ),
              );
            }
          },
        ),
      ),
    );
  }
}

